{{

  This object controls a pair of Uni-polar stepper motors connected to pins 16..19, and 20..23

}}

CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  StepWait = 500_000            'How long to wait after each motor step
  MotorSteps = 8                'How many motor steps are in the table (currently there are 8 1/2 steps, but it could be 4 whole steps)
  StepMask = MotorSteps-1       'The mask used to wrap the step counter index into the table

  XLash = 10                    'Number of steps to take to compensate for backlash when changing direction in X or Y
  YLash = 10

VAR
  long XPos, YPos               'Current position of each motor
  long XTarg, YTarg             'Desired position of each motor
  long XDelt, YDelt             'Delta to get from current to desired position
  long LastXDelt, LastYDelt     'Last delta for each axis (used to decide if we've changed direction)

  long XMotor, YMotor

PUB Init
  XPos := 0
  YPos := 0
  XMotor := 0                    'maintain these for backlash compensation
  YMotor := 0
  LastXDelt := 0
  LastYDelt := 0

  OUTA[16..19] := Masks[XMotor & StepMask] 
  OUTA[20..23] := Masks[YMotor & StepMask] 

  DIRA[16..19]~~
  DIRA[20..23]~~

  Reset


PUB Reset  
  XPos := 0
  YPos := 0

  Goto( 0, 0 )
  XPos := 0
  YPos := 0


PUB ReleaseMotors
  OUTA[16..19]~
  OUTA[20..23]~


PUB Goto( x, y ) | i

  'Store the target location
  XTarg := x
  YTarg := y

  'Compute the delta of the move
  XDelt := XTarg - XPos
  YDelt := YTarg - YPos

  'If there's no movement, just exit
  if( XDelt == 0 AND YDelt == 0 )
    return

  'Compensate for backlash if required
  DoBacklash
    
  'Figure out which principal axis we'll move on - horiz, vertical, or sloped on horiz or vert
    
  if( XDelt == 0 )
    MoveVertical
    return

  if( YDelt == 0 )
    MoveHorizontal
    return

  if( ||XDelt > ||YDelt )
    SlopeX
    return

  SlopeY


PRI DoBacklash | i
  if( XDelt <> 0 )                                      'Only care about actual moves - don't compensate if we're not moving
    if( (XDelt ~> 31) <> LastXDelt )
      if( XDelt < 0 )
        repeat i from 0 to XLash
          XMotor--
          OUTA[16..19] := Masks[ XMotor & StepMask ]
          waitcnt( StepWait + cnt )          
      else
        repeat i from 0 to XLash
          XMotor++
          OUTA[16..19] := Masks[ XMotor & StepMask ]
          waitcnt( StepWait + cnt )          
    LastXDelt := XDelt ~> 31                            'Don't remember zero deltas, just "real" direction moves                        


  if( YDelt <> 0 )       
    if( (YDelt ~> 31) <> LastYDelt )
      if( YDelt < 0 )
        repeat i from 0 to YLash
          YMotor--
          OUTA[20..23] := Masks[ YMotor & StepMask ]
          waitcnt( StepWait + cnt )          
      else
        repeat i from 0 to YLash
          YMotor++
          OUTA[20..23] := Masks[ YMotor & StepMask ]
          waitcnt( StepWait + cnt )          
     
    LastYDelt := YDelt ~> 31
     

PUB MoveVertical
  repeat
    StepY( YDelt )
    waitcnt( StepWait + cnt )          
  until YPos == YTarg


PUB MoveHorizontal
  repeat
    StepX( XDelt )
    waitcnt( StepWait + cnt )          
  until XPos == XTarg


'Function for a move primarily along X
PUB SlopeX | dx, dy, err
  dx := ||XDelt
  dy := ||YDelt
  err := dx >> 1

  repeat
    StepX( XDelt )
      err -= dy
      if err < 0
        StepY( YDelt )
        err += dx

    waitcnt( StepWait + cnt )          
         
  until( XPos == XTarg )  


'Function for a move primarily along Y
PUB SlopeY | dx, dy, err
  dx := ||XDelt
  dy := ||YDelt
  err := dy >> 1

  repeat
    StepY( YDelt )
      err -= dx
      if err < 0
        StepX( XDelt )
        err += dy
         
    waitcnt( StepWait + cnt )          

  until( YPos == YTarg )  



PUB StepX( StepDir )
  if( StepDir > 0 )
    XPos++
    XMotor++
  else
    XPos--
    XMotor--

  OUTA[16..19] := Masks[ XMotor & StepMask ]


PUB StepY( StepDir )
  if( StepDir > 0 )
    YPos++
    YMotor++
  else
    YPos--
    YMotor--

  OUTA[20..23] := Masks[ YMotor & StepMask ]

      

DAT

Masks   long  %0001,  %0011,  %0010,  %0110,  %0100,  %1100,  %1000,  %1001  
  
    
  